//-----------------------------------------------------------------------------
/*
Programmed by Olivier Sangala
http://olivier.sangala.free.fr
olivier_sangala@hotmail.com
*/
//-----------------------------------------------------------------------------

//--------------------------------------------------------------------------
// TILELIB FUNCTIONS
//--------------------------------------------------------------------------

#include <tigcclib.h>         // Include All Header Files
#include "extgraph.h"
#include "fastop.h"
#include "tilelib.h"

//--------------------------------------------------------------------------

TCamera Camera;
TPoint TileMapDim;

int scx;
int scy;

int FrameCounter=0;
int TileMasked=0;
int SpriteObject=0;

//--------------------------------------------------------------------------

char * BG_PL0=NULL;
char * BG_PL1=NULL;
char * Plane0=NULL;
char * Plane1=NULL;

unsigned char * TileMap=NULL;
unsigned short * TileData=NULL;

//--------------------------------------------------------------------------


int TL_Enable(void)
{
	
	int size=(LCD_SIZE+(32*30));
	
	if (!(BG_PL0 = malloc(2*size)))
	return 0;
	
	BG_PL1=BG_PL0+size;
	
	Plane0 = BG_PL0+(16*30);
	Plane1 = BG_PL1+(16*30);
	
	Camera.ScreenPos.x = (160/2);
	Camera.ScreenPos.y = (100/2);
	
	return 1;
	
}
//--------------------------------------------------------------------------

void TL_Disable(void)
{
	
	if (BG_PL0)
	free(BG_PL0);
	
}
//--------------------------------------------------------------------------

int CANDRAWSPRITE16(int left,int top)
{
	return (left>-16 && left<160 && top>-16 && top<100);
}
//--------------------------------------------------------------------------

void ERASE_WINDOW(void)
{
	// 92*30=2760
	// 8*30=240
	memset(Plane0+2760,0,240);
	memset(Plane1+2760,0,240);
}
//--------------------------------------------------------------------------


void DrawScreen(void)
{
	
	FrameCounter++;
	
	PlaceCamera();
	
	scx = Camera.Position.x-Camera.ScreenPos.x;
	scy = Camera.Position.y-Camera.ScreenPos.y;
	
	// BORDER CAMERA BOUNDS
	if (scx<0)
	scx=0;
	if (scy<0)
	scy=0;
	if (scx>MUL16(TileMapDim.x)-160)
	scx=MUL16(TileMapDim.x)-160;
	if (scy>MUL16(TileMapDim.y)-100+8)
	scy=MUL16(TileMapDim.y)-100+8;
	
	ClearGrayScreen2B(Plane0,Plane1);
	
	DrawBG();
	
	TileMasked=1;
	SpriteObject=1;
	
	DrawSprites();
	
	TileMasked=0;
	SpriteObject=0;
	
	DrawWindow();
	
	FastCopyScreen (Plane0,GetPlane(0));
	FastCopyScreen (Plane1,GetPlane(1));
	
}
//--------------------------------------------------------------------------

void DrawBG(void)
{
	register int x,y;
	register unsigned short * TileDataPtr;
	
	// WIDTH = 32
	register unsigned char*TileMapPtr=TileMap+MUL32(QUOT16(scy))+QUOT16(scx);
	
	register int TileX=0;
	register int TileY=-REM16(scy);
	
	register int TileCountX=(!REM16(scx))?10:11;
	register int TileCountY=(REM16(scy)<=4)?6:7;
	
	for (y=0;y<TileCountY;y++)
	{
		TileX=-REM16(scx);
		
		for (x=0;x<TileCountX;x++)
		{
			
			TileDataPtr = TileData+MUL32(*TileMapPtr);
			
			
			//if (CANDRAWSPRITE16(TileX,TileY))
			DrawTile16(TileX,TileY,TileDataPtr);
			
			TileMapPtr++;
			TileX+=16;
			
		}
		
		TileMapPtr-=TileCountX;
		TileMapPtr+=TileMapDim.x;
		
		TileY+=16;
		
	}
	
}

//--------------------------------------------------------------------------

// Function 'DrawTile8' from ExtGraph
void DrawTile8(int x,int y,unsigned char* sprite1)
{
	register long offset = (y<<5)-(y<<1)+((x>>3)&0xfffffffe);
	register long addr1  = (long)Plane0+offset;
	register long addr2  = (long)Plane1+offset;
	register unsigned int  cnt= 24-(x&15);
	register int h=8;
	
	register unsigned char* sprite2 = sprite1+8;
	
	for (;h;h--,addr1+=30,addr2+=30)
	{
		if (TileMasked)
		{
			*(long*)addr1&=~((long)(*sprite1|*sprite2)<<cnt);
			*(long*)addr2&=~((long)(*sprite1|*sprite2)<<cnt);
		}
		*(long*)addr1|=(long)(*sprite1++)<<cnt;
		*(long*)addr2|=(long)(*sprite2++)<<cnt;
	}
	
}
//--------------------------------------------------------------------------

// Function 'DrawTile16' from ExtGraph
void DrawTile16(int x,int y,unsigned short* sprite1)
{
	register long offset = (y<<5)-(y<<1)+((x>>3)&0xfffffffe);
	register long addr1  = (long)Plane0+offset;
	register long addr2  = (long)Plane1+offset;
	register unsigned int  cnt= 16-(x&15);
	register int h=16;
	
	register unsigned short* sprite2 = sprite1+16;
	
	
	for (;h;h--,addr1+=30,addr2+=30)
	{
		if (TileMasked)
		{
			*(long*)addr1&=~((long)(*sprite1|*sprite2)<<cnt);
			*(long*)addr2&=~((long)(*sprite1|*sprite2)<<cnt);
		}
		*(long*)addr1|=(long)(*sprite1++)<<cnt;
		*(long*)addr2|=(long)(*sprite2++)<<cnt;
	}
	
}
//--------------------------------------------------------------------------

void DrawSprite8(int x, int y, unsigned char * sprite1)
{
	
	if (SpriteObject)
	{
		x-=scx;
		y-=scy;
		if (CANDRAWSPRITE16(x,y))
		DrawTile8(x,y,sprite1);
	}
	else
	DrawTile8(x,y,sprite1);
	
}
//--------------------------------------------------------------------------

void DrawSprite16(int x, int y, unsigned short * sprite1)
{
	
	if (SpriteObject)
	{
		x-=scx;
		y-=scy;
		if (CANDRAWSPRITE16(x,y))
		DrawTile16(x,y,sprite1);
	}
	else
	DrawTile16(x,y,sprite1);
	
}
//--------------------------------------------------------------------------
